<?php
namespace Zeedhi\ZhuLog\Controller;

use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;

class ZhuLog {

    /** @var Manager */
    protected $dataSourceManager;
    /** @var string */
    protected $dataSourceName = 'zhuLog';

    /**
     * __construct
     *
     * @param Manager $dataSourceManager     The DataSourceManager implementation used by your product.
     */
    public function __construct(Manager $dataSourceManager) {
        $this->dataSourceManager = $dataSourceManager;
    }

    /**
     * Action used for find rows that match with given FilterCriteria
     *
     * @param DTO\Request\Filter $request  A DTO\Request with FilterCriteria to match the rows in DataSourceManager.
     * @param DTO\Response       $response A DTO\Response to be added with DataSets of matched rows.
     *
     * @return void
     */
    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();

        $conditions = array();
        foreach($filterData->getConditions() as $condition) {
            if (strpos($condition['columnName'], 'TIMESTAMP_') === 0) {
                $condition['columnName'] = 'TIMESTAMP';
            }

            $conditions[] = $condition;
        }

        $newFilter = new FilterCriteria(
            $this->dataSourceName,
            $conditions,
            $filterData->getPage(),
            $filterData->getPageSize()
        );

        $dataSet = $this->dataSourceManager->findBy($newFilter);

        $response->addDataSet($dataSet);
    }

}